@@ -33,6 +33,7 @@ class VideoImgStore(ContextManagerMixin, MultiStoreCrossIndexMixIn):
         self._selected_name = "master"
         self._index = self._master._index
         self._basedir = self._master._basedir
+        self._crossindex_pointer =0
 
     def close(self):
         for store_name in self._stores:
@@ -75,7 +76,6 @@ class VideoImgStore(ContextManagerMixin, MultiStoreCrossIndexMixIn):
             logger.debug(f"Converting grayscale image of shape {img.shape} to BGR")
             img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
 
-
         return img
 
     
@@ -88,50 +88,47 @@ class VideoImgStore(ContextManagerMixin, MultiStoreCrossIndexMixIn):
             self._metadata["imgshape"] = (self._master._metadata["imgshape"][0], self._master._metadata["imgshape"][1]*len(self._stores))
             self._index = self._selected._index
             self._make_crossindex()
+            self._stores["selected"] = self._selected
         except KeyError:
             warnings.warn(f"{name} is not a store", stacklevel=2)
+
+        
+    def get_next_image_single_store(self, store_name, fn):
+        if (self._stores[store_name].frame_number + 1) == fn:
+            with codetiming.Timer(text="get next image on master took {:.8f} seconds to compute", logger=logger.debug):
+                img, meta = self._stores[store_name].get_next_image()
+        elif (self._stores[store_name].frame_number) == fn:
+            with codetiming.Timer(text="_last_img.copy() on master took {:.8f} seconds to compute", logger=logger.debug):
+                img = self._stores[store_name]._last_img.copy()
+                meta = (self._stores[store_name].frame_number, self._stores[store_name].frame_time)
+        else:
+            with codetiming.Timer(text="get image on master took {:.8f} seconds to compute", logger=logger.debug):
+                try:
+                    img, meta = self._stores[store_name].get_image(fn)
+                except Exception as error:
+                    raise error
+
+        logger.debug(f"{store_name} is in frame number #{self._stores[store_name].frame_number}")
+        return img, meta
+
     
     def get_next_image(self):
+        
+        id=self._crossindex_pointer+1
 
-        img, meta = self._selected.get_next_image()
-        master_fn = self.crossindex.find_master_fn(self._selected.frame_number)
-        frame_number, frame_time = meta
+        master_fn = self.crossindex.find_fn_given_id(store="master", id=id)
+        selected_fn = self.crossindex.find_fn_given_id(store="selected", id=id)
+        
+        master_img, master_meta = self.get_next_image_single_store("master", master_fn)
+        selected_img, selected_meta = self.get_next_image_single_store("selected", selected_fn)
 
-        if conf.LOOKUP_NEAREST:
-            with codetiming.Timer(text="get nearest image on master took {:.8f} seconds to compute", logger=logger.debug):
-                master_img, _ = self._master.get_nearest_image(frame_time)
-        else:
-            if (self._master.frame_number + 1) == master_fn:
-                with codetiming.Timer(text="get next image on master took {:.8f} seconds to compute", logger=logger.debug):
-                    master_img, _ = self._master.get_next_image()
-            elif (self._master.frame_number) == master_fn:
-                with codetiming.Timer(text="_last_img.copy() on master took {:.8f} seconds to compute", logger=logger.debug):
-                    master_img = self._master._last_img.copy()
-            else:
-                with codetiming.Timer(text="get image on master took {:.8f} seconds to compute", logger=logger.debug):
-                    try:
-                        master_img, _ = self._master.get_image(master_fn)
-                    except Exception as error:
-                        raise error
-
-        print(f"Master is in frame number #{self._master.frame_number}")
-        print(f"Selected is  in frame number #{self._selected.frame_number}")
-
-        if self._selected_name == "master":
-            imgs = [master_img]
-        else:
-            imgs = [master_img, img]
-
-        for store_name, store in self._stores.items():
-            if store_name in ["master", self._selected_name]:
-                continue
-            else:
-                with codetiming.Timer(text="get nearest image on {store_name} took {:.8f} seconds to compute", logger=logger.debug):
-                    img, _ = store.get_nearest_image(frame_time)
-                    imgs.append(img)
+        imgs=[master_img, selected_img]
 
         with codetiming.Timer(text="_apply_layout took {:.8f} seconds to compute", logger=logger.debug):
             img = self._apply_layout(imgs)
+        
+        meta = (id, selected_meta[1])
+        self._crossindex_pointer =id
         return img, meta
 
     
@@ -149,26 +146,38 @@ class VideoImgStore(ContextManagerMixin, MultiStoreCrossIndexMixIn):
 
 
     def get_next_framenumber(self):
+        # TODO
+        # Can the idx be None
 
-        frame_number, idx = self._selected.get_next_framenumber()
+        frame_number, idx = (self._crossindex_pointer, None)
         return frame_number, idx
 
     def get_nearest_image(self, frame_time):
+        try:
+            master_img, master_meta = self._master.get_nearest_image(frame_time, future=False)
+        except TypeError:
+            logger.warning(f"Cannot fetch a frame in master before {frame_time}. Fetching from future")
+            master_img, master_meta = self._master.get_nearest_image(frame_time)
+
+        selected_meta = master_meta
+
+        self._crossindex_pointer = self.crossindex.find_id_given_fn("master", master_meta[0])
+        selected_frame_number=self.crossindex.find_selected_fn(master_meta[0])
 
-        master_img, selected_meta = self._master.get_nearest_image(frame_time)
         imgs = [master_img]
 
         for store_name, store in self._stores.items():
             if store_name == "master":
                 continue
             else:
-                img, meta = store.get_nearest_image(frame_time)
+                img, meta = store.get_image(selected_frame_number)
                 if store_name == self._selected_name:
                     selected_meta = meta
 
             imgs.append(img)
 
         img = self._apply_layout(imgs)
+        
         return img, selected_meta
 
     def get_image(self, frame_number, exact_only=True, frame_index=None):
@@ -181,42 +190,99 @@ class VideoImgStore(ContextManagerMixin, MultiStoreCrossIndexMixIn):
         :param frame_index: frame_index (0, frame_count]
         """
 
+        id=frame_number
+        del frame_number
+
         if _VERBOSE_DEBUG_GETS:
-            self._log.debug('get_image %s (exact: %s) frame_idx %s' % (frame_number, exact_only, frame_index))
+            self._log.debug('get_image %s (exact: %s) frame_idx %s' % (id, exact_only, frame_index))
         
         imgs=[None, None]
-        # master_fn = self.crossindex.loc[frame_number, ("master", "frame_number")]
         with codetiming.Timer(text="find_master_fn took {:.8f} seconds to compute", logger=logger.debug):
-            master_fn = self.crossindex.find_master_fn(frame_number)
+            master_fn = self.crossindex.find_fn_given_id("master", id)
+
 
         with codetiming.Timer(text="get_image on master took {:.8f} seconds to compute", logger=logger.debug):
             imgs[0], _ = self._master.get_image(master_fn)
         with codetiming.Timer(text="get_image on selected took {:.8f} seconds to compute", logger=logger.debug):
-            imgs[1], selected_meta = self._selected.get_image(frame_number)
+            selected_fn = self.crossindex.find_fn_given_id("selected", "id") 
+            imgs[1], selected_meta = self._selected.get_image(selected_fn)
+
 
         img = self._apply_layout(imgs)
+        self._crossindex_pointer = id
         return img, selected_meta
 
-        # chunk, frame_idx, frame_number, frame_time = self._index.find_all("frame_number", frame_number, exact_only=exact_only)
-        # return self.get_nearest_image(frame_time)
-
 
     def get_chunk(self, chunk):
         self._log.debug(f"{self}.get_chunk({chunk})")
-        _, (fn, ft) = self._master.get_chunk(chunk)
+        _, (master_fn, master_ft) = self._master.get_chunk(chunk)
 
         for store_name, store in self._stores.items():
             if store_name == "master":
                 continue
             else:
-                img, (fn, ft) = store.get_nearest_image(ft)
+                with codetiming.Timer():
+                    id=self.crossindex.find_id_given_fn("master", master_fn)
+                    selected_fn=self.crossindex.find_fn_given_id("selected", id)
+                    img, (fn, ft) = self.get_image(selected_fn)
+                # This is a bit slower
+                # with codetiming.Timer():
+                #     img2, (fn2, ft2) = self.get_nearest_image(master_ft)
+
+
                 store.get_image(max(0, fn))
+        self._crossindex_pointer = self.crossindex.find_id_given_fn("master", master_fn)
 
 
     @classmethod
     def new_for_filename(cls, *args, **kwargs):
         return new_for_filename(*args, **kwargs)
 
+    def set(self, key, value):
+        result=self._selected.set(key, value)
+        if key == 0:
+            self.get_nearest_image(value-1)
+        elif key == 1:
+            self._crossindex_pointer=value
+            self.get_image(max(value-1, 0))
+
+        return result
+
+
+    def get(self, property):
+
+        if property == "NUMBER_OF_FRAMES_IN_CHUNK":
+            # this is the number of frames in the chunk
+            chunk = self._master._chunk_n
+            return self.crossindex.get_number_of_frames_in_chunk(chunk)
+            
+
+        elif property == "NUMBER_OF_FRAMES":
+            # this is the number of frames in the store up to the last one
+            # in the current chunk
+            raise NotImplementedError()
+
+
+        elif property == "STARTING_FRAME_OF_CHUNK":
+            # this is the frame number of the last frame in this chumk
+            chunk = self._master._chunk_n
+            return self.crossindex.get_starting_frame_of_chunk(chunk)
+         
+
+        elif property == "ENDING_FRAME_OF_CHUNK":
+            # this is the frame number of the last frame in this chumk
+            chunk = self._master._chunk_n
+            return self.crossindex.get_ending_frame_of_chunk(chunk)
+
+        elif property == "TOTAL_NUMBER_OF_FRAMES":
+            return self.crossindex.get_number_of_frames()
+
+        elif property == 1:
+            return self._crossindex_pointer
+
+        else:
+            return self._selected.get(property)
+
     def __getattr__(self, __name: str):
         return getattr(self._selected, __name)
 
